% dataset = '';
% load(dataset);

robot = 'ergoCubSN000';

gyro_name = 'r_foot_front_ft_gyro';
acc_name = 'r_foot_front_ft_acc';
imu_frame = 'r_foot_front_ft_sensor';

robot_joints = ["r_hip_pitch", "r_hip_roll", "r_hip_yaw", "r_knee", "r_ankle_pitch", "r_ankle_roll"];
for joint = 1 : length(robot_joints)
    joint_indeces(joint) = find(strcmp(robot_logger_device.description_list, robot_joints(joint)));
end

model_loader = iDynTree.ModelLoader();
considere_joints = iDynTree.StringVector();
for idx = 1 : length(robot_joints)
considere_joints.push_back(robot_joints{idx});
end
for idx = 1 : length(robot_fts)
considere_joints.push_back(robot_fts{idx});
end
model_loader.loadReducedModelFromFile(strcat('./urdf/', robot,'.urdf'), considere_joints);
disp('Model loaded')
% disp(model_loader.model().toString())

grav_idyn = iDynTree.Vector3();
grav = [0.0; 0.0; -9.81];
grav_idyn.fromMatlab(grav);
base_acceleration_idyn = iDynTree.Vector6();
base_acceleration_idyn.zero();
% Define kindyn object
kinDyn = iDynTree.KinDynComputations();
kinDyn.loadRobotModel(model_loader.model());

kinDyn.setFrameVelocityRepresentation(iDynTree.BODY_FIXED_REPRESENTATION);

gyroscope = [];
accelerometer = [];
positions = squeeze(robot_logger_device.joints_state.positions.data)';
velocities = squeeze(robot_logger_device.joints_state.velocities.data)';
accelerations = squeeze(robot_logger_device.joints_state.accelerations.data)';
for joint = 1 : length(robot_joints)
    joint_indeces(joint) = find(strcmp(robot_logger_device.description_list, robot_joints(joint)));
end

% Filter velocities and accelerations
x0 = [positions(1,:), zeros(1, length(robot_logger_device.description_list)*2)]';
q = [repmat(1e-2, length(robot_logger_device.description_list), 1); repmat(1e-1, length(robot_logger_device.description_list), 1); repmat(1e0, length(robot_logger_device.description_list), 1)];
r = repmat(1e-5, length(robot_logger_device.description_list), 1);

kf = initKFNullJerk(x0, q, r, 0.01);

joint_positions_kf = [];
joint_velocities_kf = [];
joint_accelerations_kf = [];
for sample = 1 : size(positions, 1)
    kf.z = positions(sample, :)';
    kf = kalmanfilter(kf);
    positions(sample,:) = kf.x(1:length(robot_logger_device.description_list));
    velocities(sample,:) = kf.x(length(robot_logger_device.description_list)+1:length(robot_logger_device.description_list)*2);
    accelerations(sample,:) = kf.x(length(robot_logger_device.description_list)*2+1:end);
end

s_idyn = iDynTree.JointPosDoubleArray(kinDyn.model);
ds_idyn = iDynTree.JointDOFsDoubleArray(kinDyn.model);
dds_idyn = iDynTree.JointDOFsDoubleArray(kinDyn.model);
baseAcc = iDynTree.Vector6;
baseAcc.zero();
% Compute joint torque
for sample = 1 : size(positions,1)
    s_idyn.fromMatlab(positions(sample, joint_indeces));
    ds_idyn.fromMatlab(velocities(sample, joint_indeces));
    dds_idyn.fromMatlab(accelerations(sample, joint_indeces));

    kinDyn.setRobotState(s_idyn, ds_idyn, grav_idyn);
    gyroscope(sample, :) = kinDyn.getFrameVel(imu_frame).toMatlab();
    accelerometer(sample, :) = kinDyn.getFrameAcc(imu_frame, baseAcc, dds_idyn).toMatlab();

    rot = kinDyn.getWorldTransform(imu_frame).getRotation().inverse();
    accelerometer(sample, 1:3) = accelerometer(sample, 1:3) - (rot.toMatlab() * grav_idyn.toMatlab())';
end

gyro_logged = squeeze(robot_logger_device.gyros.(gyro_name).data)';
acc_logged = squeeze(robot_logger_device.accelerometers.(acc_name).data)';


% Plot gyroscope

labels = {'\omega_x', '\omega_y', '\omega_z'};
figure,
for i = 1 : 3
subplot(3,1,i),
plot(gyro_logged(:,i))
hold on
plot(gyroscope(:, i + 3))
legend('gyroscope', 'kinematics')
xlabel('samples')
ylabel(strcat(labels{i}, ' (rad/sec)'))
end
sgtitle(gyro_name, 'Interpreter', 'none') 

figure,
subplot(3,1,1),
plot(gyro_logged(:,1))
hold on
plot(-gyroscope(:, 5))
legend('gyroscope (-y)', 'kinematics')
xlabel('samples')
ylabel('\omega_x (rad/sec)')
subplot(3,1,2),
plot(gyro_logged(:,2))
hold on
plot(-gyroscope(:, 4))
legend('gyroscope (-x)', 'kinematics')
xlabel('samples')
ylabel('\omega_y (rad/sec)')
subplot(3,1,3),
plot(gyro_logged(:,3))
hold on
plot(-gyroscope(:, 6))
legend('gyroscope (-z)', 'kinematics')
xlabel('samples')
ylabel('\omega_z (rad/sec)')
sgtitle(gyro_name, 'Interpreter', 'none')


% Plot accelerometer
labels = {'dv_x', 'dv_y', 'dv_z'};
figure,
for i = 1 : 3
subplot(3,1,i),
plot(acc_logged(:,i))
hold on
plot(accelerometer(:, i))
legend('accelerometer', 'kinematics')
xlabel('samples')
ylabel(strcat(labels{i}, ' (rad/sec^2)'))
end
sgtitle(acc_name, 'Interpreter', 'none') 

figure,
subplot(3,1,1),
plot(acc_logged(:,1))
hold on
plot(-accelerometer(:, 2))
legend('accelerometer (-y)', 'kinematics')
xlabel('samples')
ylabel('dv_x (rad/sec^2)')
subplot(3,1,2),
plot(acc_logged(:,2))
hold on
plot(-accelerometer(:, 1))
legend('accelerometer (-x)', 'kinematics')
xlabel('samples')
ylabel('dv_y (rad/sec^2)')
subplot(3,1,3),
plot(acc_logged(:,3))
hold on
plot(-accelerometer(:, 3))
legend('accelerometer (-z)', 'kinematics')
xlabel('samples')
ylabel('dv_z (rad/sec^2)')
sgtitle(acc_name, 'Interpreter', 'none')